@@ -96,9 +96,14 @@ class CryptoCompare:
         return output
 
 
-def binance_get_histo_future_balance(api, p, request_date):
+def binance_get_histo_future_balance_snapshot(api, p, request_date):
     bot = binance_bot.NaPoleonBinanceFutureBot(api, p)
-    return bot.get_balance_snapshot(request_date)
+    return bot.get_futures_balance_snapshot(request_date)
+
+def binance_get_future_balance_history(api, p, start_time, end_time):
+    bot = binance_bot.NaPoleonBinanceFutureBot(api, p)
+    return bot.get_futures_balance_history(start_time, end_time)
+
 
 def binance_get_futures_transfer_histo(api, p, start_time, end_time):
     bot = binance_bot.NaPoleonBinanceFutureBot(api, p)
@@ -429,25 +434,35 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
     balance_pkl_file_name = f'{filename_stub}_balance.pkl'
     transfer_pkl_file_name = f'{filename_stub}_transfer.pkl'
 
-    initial_balance_time = start_time
-    initial_balances = None
-    while (initial_balances is None or initial_balances.empty) and not initial_balance_time>end_time:
-        try:
-            initial_balances = binance_get_histo_future_balance(public_key, private_key, initial_balance_time)
-        except Exception as e:
-            print(e)
-        initial_balance_time = initial_balance_time + timedelta(days = 1)
 
+    ohlc_bnb_min_binance = binance_get(public_key, private_key, 'spot_klines', start_time, end_time=end_time, symbol ='BNBUSDT', aggreg ='1m')
+
+    history_balance = binance_get_future_balance_history(public_key,private_key,start_time, end_time)
+
+    history_balance = pd.merge(history_balance, ohlc_bnb_min_binance[['ts','open']], how='left', left_on = 'date', right_on = 'ts')
+
+
+    def convert_bnb_to_usdt(row):
+        print(row)
+        if row['asset'] == 'BNB':
+            return float(row['walletBalance']) * float(row['open'])
+        elif row['asset'] == 'USDT':
+            return float(row['walletBalance'])
+        else :
+            return np.nan
+
+
+    history_balance['convertedWalletBalance'] = history_balance.apply(convert_bnb_to_usdt, axis=1)
+
+    converted_history_balance = history_balance[['ts','convertedWalletBalance']].groupby('ts').sum()
 
 
     fut_transfers = binance_get_futures_transfer_histo(public_key, private_key, start_time, end_time)
-
     trades = binance_get(public_key, private_key, 'trade_history', start_time, end_time=end_time, symbol ='ETHUSDT').drop_duplicates()
     ohlc_eth_binance = binance_get(public_key, private_key, 'futures_klines', start_time, end_time= end_time, symbol ='ETHUSDT', aggreg ='1m')
     ohlc_btc_binance = binance_get(public_key, private_key, 'futures_klines', start_time, end_time= end_time, symbol ='BTCUSDT', aggreg ='1m')
     walletHist = binance_get(public_key, private_key, 'wallet_history', start_time, end_time= end_time, symbol ='ETHUSDT')
 
-    ohlc_bnb_min_binance = binance_get(public_key, private_key, 'spot_klines', start_time, end_time=end_time, symbol ='BNBUSDT', aggreg ='1m')
 
 #    initial_balance_time =  min(pd.to_datetime(trades['time'], unit='ms'))
 
@@ -462,7 +477,7 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
 
     if save_to_dropbox :
         full_path = local_root_directory + balance_pkl_file_name
-        initial_balances.to_pickle(full_path)
+        converted_history_balance.to_pickle(full_path)
         print(f'uploading to dropbox {full_path}')
         dbx.upload(fullname=full_path, folder='', subfolder='', name=balance_pkl_file_name, overwrite=True)
         os.remove(full_path)
@@ -646,22 +661,10 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
         wh = pd.merge(wh, fut_transfers_usd, how='left', left_index=True, right_index=True)
         wh = pd.merge(wh, fut_transfers_bnb, how='left', left_index=True, right_index=True)
 
-    initial_bnb_balance = 0
-    initial_usdt_balance = 0
-    try:
-        initial_bnb_balance = float(initial_balances[initial_balances['asset'] == 'BNB']['walletBalance'].iloc[0])
-    except Exception as e:
-        print(e)
-    try:
-        initial_usdt_balance = float(initial_balances[initial_balances['asset'] == 'USDT']['walletBalance'].iloc[0])
-    except Exception as e:
-        print(e)
+    wh['date'] = wh.index
+    wh = pd.merge(wh, converted_history_balance, how='left', left_on='date', right_on ='ts')
 
-    bnb_rate = float(ohlc_bnb_min_binance.loc[start_time]['open_bnb_bnc'])
-    initial_usd_balance = initial_usdt_balance + initial_bnb_balance * bnb_rate
-    print(f'initial usdt balance {initial_usd_balance}')
-
-    wh.at[wh.index[0], 'realized_pnl'] = initial_usd_balance
+    wh.at[wh.index[0], 'realized_pnl'] = wh.at[wh.index[0], 'convertedWalletBalance']
 
     wh['net_profit'] = wh.realized_pnl + wh.funding_fees_eth_usd + wh.funding_fees_xbt_usd + wh.trading_fees_eth_usd + wh.trading_fees_xbt_usd
     wh['valuation'] = wh.net_profit + wh.transfers_usd
